# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir surveillant le fonctionnement du routeur
"""

from twisted.internet.utils import getProcessOutput

from creole import parsedico

from zephir.monitor.agentmanager.util import log
from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import TableData
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from zephir.monitor.agentmanager import status

class Passerelle(Agent):

    def __init__(self, name, **params):

        Agent.__init__(self, name, **params)
        try:
            d = parsedico.parse_dico()
            self.adresse_gw = d['adresse_ip_gw']
        except:
            self.adresse_gw = ""
        
        self.table = TableData([
            ('status', "état", {'align':'center'}, status_to_img), 
            ('description', "Description", {'align':'left'}, None),
            ('adresse', "Adresse", {'align':'left'}, None) ])
        self.data = [self.table]

    def measure(self):
        res = getProcessOutput("/usr/sbin/fping", 
                               args = [self.adresse_gw],
                               env = {'LC_ALL': 'C'})
        return res.addCallbacks(self.measure_fping, self.error_process)
            
    def measure_fping(self,data):
        # fping est sorti sans erreur
        mesures=[{ 'description': 'Contact avec la passerelle',
                   'adresse': self.adresse_gw,
                   'status': 'On' }]
        return mesures

    def error_process(self, *args):
        # fping a renvoyé un code d'erreur
        self.measure_data['gw'] = True
        mesures=[{ 'description': 'Contact avec la passerelle',
                   'adresse': self.adresse_gw,
                   'status': 'Off'}]
        return mesures

    def write_data(self):
        self.measure_data['gw'] = False
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            if self.last_measure.value is not None:
                for service in self.last_measure.value:
                    if service['status'] != 'On':
                        return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

